@@ -1,4 +1,4 @@
-# $Id: Session.pm,v 1.48 2000/06/15 17:16:51 rcaputo Exp $
+# $Id: Session.pm,v 1.51 2000/11/17 22:33:17 rcaputo Exp $
 
 package POE::Session;
 
@@ -18,6 +18,7 @@ const CREATE_OPTIONS  'options'
 const CREATE_INLINES  'inline_states'
 const CREATE_PACKAGES 'package_states'
 const CREATE_OBJECTS  'object_states'
+const CREATE_HEAP     'heap'
 
 const OPT_TRACE   'trace'
 const OPT_DEBUG   'debug'
@@ -415,6 +416,13 @@ sub create {
       }
     }
 
+    # Import an external heap.  This is a convenience, since it
+    # eliminates the need to connect _start options to heap values.
+
+    elsif ($param_name eq CREATE_HEAP) {
+      $self->[SE_NAMESPACE] = $param_value;
+    }
+
     else {
       croak "unknown $type parameter: $param_name";
     }
@@ -645,6 +653,15 @@ sub option {
   }
 }
 
+# Fetch the session's heap.  In rare cases, libraries may need to
+# break encapsulation this way, probably also using
+# $kernel->get_current_session as an accessory to the crime.
+
+sub get_heap {
+  my $self = shift;
+  return $self->[SE_NAMESPACE];
+}
+
 #------------------------------------------------------------------------------
 # Create an anonymous sub that, when called, posts an event back to a
 # session.  This is highly experimental code to support Tk widgets and
@@ -679,7 +696,8 @@ sub postback {
 
   my $postback = bless
     sub {
-      $POE::Kernel::poe_kernel->post( $id, $event, [ @etc ], [ @_ ] )
+      $POE::Kernel::poe_kernel->post( $id, $event, [ @etc ], [ @_ ] );
+      0;
     }, 'POE::Session::Postback';
 
   $postback_parent_id{$postback} = $id;
@@ -777,6 +795,9 @@ them.  The create() constructor is therefore recommended over new().
     # session options to be set at creation time.  'options' refers to
     # a hash containing option names and initial values.
     options => \%options,
+
+    # Specify a heap other than the default empty hashref.
+    heap => [ ],
   );
 
 Other methods:
@@ -784,6 +805,9 @@ Other methods:
   # Retrieve a session's unique identifier.
   $session_id = $session->ID;
 
+  # Retrieve a reference to the session's heap.
+  $session_heap = $session->get_heap;
+
   # Set or clear session options.
   $session->option( trace => 1, default => 1 );
   $session->option( trace );
@@ -1280,6 +1304,28 @@ passed in as @_[ARG0..$#_].
 
   args => [ 'arg0', 'arg1', 'etc.' ],
 
+=item heap => ANYTHING or NOTHING
+
+Defines the session's heap, which will be passed to each of its states
+in $_[HEAP].  Sessions are created with anonymous hash references by
+default, which are used like this:
+
+  my $heap = $_[HEAP];
+  $heap->{variable} = $value;  # simple hashref stuff
+
+It's possible, however, to use create's C<heap> parameter to change
+that into a list reference, or even some sort of magical tied thing.
+It's also possible to pre-populate a session's heap.
+
+  POE::Session->create(
+    ...,
+    heap => [ 'mi',    # HEAP_ALIAS ( These are constants defined
+              [ ],     # HEAP_QUEUE   elsewhere.
+              0,       # HEAP_COUNT )
+            ],
+    ....,
+  );
+
 =item inline_states => HASHREF
 
 Defines inline coderefs that make up some or all of the session's
@@ -1427,6 +1473,38 @@ Event->flawor gives its callback.
       desc => 'post ev_flavor when Event->flavor occurs',
     );
 
+=item get_heap
+
+Returns a reference to the session's heap.  This is the same value
+that's passed to the session in $_[HEAP].  The difference is that this
+method may be used by functions which are expected to be called by a
+session but which don't want the heap to be passed to them.  In those
+cases, the function would use $poe_kernel->get_active_session() first
+to determine the session context in which it was called:
+
+  sub use_a_wheel {
+    my @stuff_to_put = @_;
+    $poe_kernel->get_active_session()->heap()->{wheel}->put(@stuff_to_put);
+  }
+
+It probably is more efficient for sessions to pass $_[HEAP] along:
+
+  sub use_a_wheel {
+    my ($heap, @stuff_to_put) = @_;
+    $heap->{wheel}->put(@stuff_to_put);
+  }
+
+Or even:
+
+  sub use_a_wheel {
+    my $heap = shift;
+    $heap->{wheel}->put(@_);
+  }
+
+Although if you expect to have a lot of calls to &put_a_wheel() in
+your program, you may want to optimize for programmer efficiency by
+using the first form.
+
 =back
 
 =head1 SEE ALSO
